#include "../../../lib/std.mi"

Global Layer layerNormalSeekSurface, layerNormalSeekGhost;
Global Layer layerClassicSeekSurface, layerClassicSeekGhost;
Global Layer layerStickSeekSurface, layerStickSeekGhost;
Global Layer layerMiniSeekSurface, layerMiniSeekGhost;
Global Map mapNormalSeek, mapClassicSeek, mapStickSeek, mapMiniSeek;
Global Int intSeekClicked;
Global Timer timerSeek, timerSongTicker;
Global Int intCurrentPosition;
Global Text textNormalSongInfo, textClassicSongInfo, textStickSongInfo;
Global Text textNormalSeekInfo, textClassicSeekInfo, textStickSeekInfo;

Function seekInit();
Function seekUpdate(Int intNewPosition);
Function seekUpdateUtil(Int intNewPosition, Map mapSeek, Layer layerSeekGhost);
Function seekUpdateXY(Int intPosX, Int intPosY, Map mapSeek, Layer layerSeekGhost, Text textSongTicker, Text textSeekTicker);
Function seekMoveTo(Int intPosX, Int intPosY, Map mapSeek);
Function layerSeekSurfaceLeftButtonDownUtil(Int intPosX, Int intPosY, Map mapSeek, Layer layerSeekGhost, Text textSongTicker, Text textSeekTicker);
Function layerSeekSurfaceMouseMoveUtil(Int intPosX, Int intPosY, Map mapSeek, Layer layerSeekGhost, Text textSongTicker, Text textSeekTicker);
Function layerSeekSurfaceLeftButtonUpUtil(Int intPosX, Int intPosY, Map mapSeek);

System.onScriptLoaded() {
	seekInit();
}

System.onScriptUnloading() {
	delete timerSeek;
	delete timerSongTicker;
}

seekInit() {
	Layout layoutMainNormal = System.getContainer("main").getLayout("normal");
	Layout layoutMainClassic = System.getContainer("main").getLayout("classic");
	Layout layoutMainStick = System.getContainer("main").getLayout("stick");
	Layout layoutMainMini = System.getContainer("main").getLayout("mini");
	
	Group groupNormalDisplay = layoutMainNormal.getObject("player.normal.group.display");
	Group groupClassicDisplay = layoutMainClassic.getObject("player.classic.group.display");
	Group groupStickDisplay = layoutMainStick.getObject("player.stick.group.display");
	Group groupMiniDisplay = layoutMainMini.getObject("player.mini.group.display");
	
	layerNormalSeekSurface = groupNormalDisplay.findObject("PlayerSeekTrack");
	layerNormalSeekGhost = groupNormalDisplay.findObject("PlayerSeekHandle");
	textNormalSongInfo = groupNormalDisplay.getObject("PlayerInfoLine");
	textNormalSeekInfo = groupNormalDisplay.getObject("PlayerSeekInfo");
	
	layerClassicSeekSurface = groupClassicDisplay.findObject("PlayerSeekTrack");
	layerClassicSeekGhost = groupClassicDisplay.findObject("PlayerSeekHandle");
	textClassicSongInfo = groupClassicDisplay.getObject("PlayerInfoLine");
	textClassicSeekInfo = groupClassicDisplay.getObject("PlayerSeekInfo");
	
	layerStickSeekSurface = groupStickDisplay.findObject("PlayerSeekTrack");
	layerStickSeekGhost = groupStickDisplay.findObject("PlayerSeekHandle");
	textStickSongInfo = groupStickDisplay.getObject("PlayerSongTicker");
	textStickSeekInfo = groupStickDisplay.getObject("PlayerSeekInfo");
	
	layerMiniSeekSurface = groupMiniDisplay.findObject("PlayerSeekTrack");
	layerMiniSeekGhost = groupMiniDisplay.findObject("PlayerSeekHandle");
	
	mapNormalSeek = new Map;
	mapClassicSeek = new Map;
	mapStickSeek = new Map;
	mapMiniSeek = new Map;
	mapNormalSeek.loadMap("player.normal.display.seek.map");
	mapClassicSeek.loadMap("player.classic.display.seek.map");
	mapStickSeek.loadMap("player.stick.display.seek.map");
	mapMiniSeek.loadMap("player.mini.display.seek.map");
	
	textNormalSeekInfo.hide();
	textClassicSeekInfo.hide();
	textStickSeekInfo.hide();
	
	seekUpdate(0);
	
	timerSeek = new Timer;
	timerSeek.setDelay(150);
	timerSeek.start();
	
	timerSongTicker = new Timer;
}

seekUpdate(Int intNewPosition) {
	seekUpdateUtil(intNewPosition, mapNormalSeek, layerNormalSeekGhost);
	seekUpdateUtil(intNewPosition, mapClassicSeek, layerClassicSeekGhost);
	seekUpdateUtil(intNewPosition, mapStickSeek, layerStickSeekGhost);
	seekUpdateUtil(intNewPosition, mapMiniSeek, layerMiniSeekGhost);
}

seekUpdateUtil(Int intNewPosition, Map mapSeek, Layer layerSeekGhost) {
	Float floatPosition;
	Int intPlayItemLength = System.getPlayItemLength();
	if (intPlayItemLength == 0) {
		floatPosition = 0;
	} else {
		floatPosition = intNewPosition / intPlayItemLength * 255;
	}
	Region regionPosition = new Region;
	regionPosition.loadFromMap(mapSeek, floatPosition, 1);
	intCurrentPosition = floatPosition;
	regionPosition.offset(-layerSeekGhost.getLeft(), -layerSeekGhost.getTop());
	layerSeekGhost.setRegion(regionPosition);
	delete regionPosition;
}

seekUpdateXY(Int intPosX, Int intPosY, Map mapSeek, Layer layerSeekGhost, Text textSongTicker, Text textSeekTicker) {
	Int intPosition = mapSeek.getValue(intPosX, intPosY);
	Region regionPosition = new Region;
	regionPosition.loadFromMap(mapSeek, intPosition, 1);
	regionPosition.offset(-layerSeekGhost.getLeft(), -layerSeekGhost.getTop());
	layerSeekGhost.setRegion(regionPosition);
	delete regionPosition;
	
	Float floatPositionPercentage = intPosition / 255 * 100;
	Float floatPlayItemLength = System.getPlayItemLength();
	Int intNewPosition = floatPlayItemLength * floatPositionPercentage / 100;
	timerSongTicker.stop();
	timerSongTicker.start();
	
	textSongTicker.hide();
	textSeekTicker.show();
	textSeekTicker.setText("Seek to " + System.integerToTime(intNewPosition) + " / " + System.integerToTime(floatPlayItemLength) + " (" + System.integerToString(floatPositionPercentage) + "%)");
}

timerSongTicker.onTimer() {
	if (!intSeekClicked) {
		textNormalSeekInfo.hide();
		textClassicSeekInfo.hide();
		textStickSeekInfo.hide();
		
		textNormalSongInfo.show();
		textClassicSongInfo.show();
		textStickSongInfo.show();
	}
}

seekMoveTo(Int intPosX, Int intPosY, Map mapSeek) {
	Int intPosition = mapSeek.getValue(intPosX, intPosY);
	System.seekTo(System.getPlayItemLength() * (intPosition / 255));
}

layerNormalSeekSurface.onLeftButtonDown(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonDownUtil(intPosX, intPosY, mapNormalSeek, layerNormalSeekGhost, textNormalSongInfo, textNormalSeekInfo);
}
layerClassicSeekSurface.onLeftButtonDown(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonDownUtil(intPosX, intPosY, mapClassicSeek, layerClassicSeekGhost, textClassicSongInfo, textClassicSeekInfo);
}
layerStickSeekSurface.onLeftButtonDown(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonDownUtil(intPosX, intPosY, mapStickSeek, layerStickSeekGhost, textStickSongInfo, textStickSeekInfo);
}
layerMiniSeekSurface.onLeftButtonDown(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonDownUtil(intPosX, intPosY, mapMiniSeek, layerMiniSeekGhost, textStickSongInfo, textStickSeekInfo);
}
layerSeekSurfaceLeftButtonDownUtil(Int intPosX, Int intPosY, Map mapSeek, Layer layerSeekGhost, Text textSongTicker, Text textSeekTicker) {
	if (System.getPlayItemLength() <= 0) return;
	if (strLeft(System.getPlayItemString(), 4) == "http") return;
	intSeekClicked = 1;
	seekUpdateXY(intPosX, intPosY, mapSeek, layerSeekGhost, textSongTicker, textSeekTicker);
}

layerNormalSeekSurface.onMouseMove(Int intPosX, Int intPosY) {
	layerSeekSurfaceMouseMoveUtil(intPosX, intPosY, mapNormalSeek, layerNormalSeekGhost, textNormalSongInfo, textNormalSeekInfo);
}
layerClassicSeekSurface.onMouseMove(Int intPosX, Int intPosY) {
	layerSeekSurfaceMouseMoveUtil(intPosX, intPosY, mapClassicSeek, layerClassicSeekGhost, textClassicSongInfo, textClassicSeekInfo);
}
layerStickSeekSurface.onMouseMove(Int intPosX, Int intPosY) {
	layerSeekSurfaceMouseMoveUtil(intPosX, intPosY, mapStickSeek, layerStickSeekGhost, textStickSongInfo, textStickSeekInfo);
}
layerMiniSeekSurface.onMouseMove(Int intPosX, Int intPosY) {
	layerSeekSurfaceMouseMoveUtil(intPosX, intPosY, mapMiniSeek, layerMiniSeekGhost, textStickSongInfo, textStickSeekInfo);
}
layerSeekSurfaceMouseMoveUtil(Int intPosX, Int intPosY, Map mapSeek, Layer layerSeekGhost, Text textSongTicker, Text textSeekTicker) {
	if (intSeekClicked) {
		if (System.getPlayItemLength() == 0) {
			intSeekClicked = 0;
			return;
		}
		seekUpdateXY(intPosX, intPosY, mapSeek, layerSeekGhost, textSongTicker, textSeekTicker);
	}
}

layerNormalSeekSurface.onLeftButtonUp(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonUpUtil(intPosX, intPosY, mapNormalSeek);
}
layerClassicSeekSurface.onLeftButtonUp(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonUpUtil(intPosX, intPosY, mapClassicSeek);
}
layerStickSeekSurface.onLeftButtonUp(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonUpUtil(intPosX, intPosY, mapStickSeek);
}
layerMiniSeekSurface.onLeftButtonUp(Int intPosX, Int intPosY) {
	layerSeekSurfaceLeftButtonUpUtil(intPosX, intPosY, mapMiniSeek);
}
layerSeekSurfaceLeftButtonUpUtil(Int intPosX, Int intPosY, Map mapSeek) {
	if (!intSeekClicked) return;
	intSeekClicked = 0;
	seekMoveTo(intPosX, intPosY, mapSeek);
}

timerSeek.onTimer() {
	if (intSeekClicked) return;
	Int intPlayItemLength = System.getPlayItemLength();
	if (intPlayItemLength > 0) {
		Int intPosition = System.getPosition() / intPlayItemLength * 255;
		if (intPosition != intCurrentPosition) {
			seekUpdate(System.getPosition());
		}
	} else {
		if (intCurrentPosition != 0) {
			seekUpdate(0);
			intCurrentPosition = 0;
		}
	}
}